// $Id: MainWindow.cpp,v 1.40 2013/03/07 14:05:19 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.	See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along Mwith
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA	02111-1307 USA
*
*
*******************************************************************************/

#include "MainWindow.h"

#include "Application.h"
#include "BaseIcons.h"
#include "Icons.h"
#include "IconEngine.h"
#include "Menu.h"
#include "QtUtil.h"
#include "Singleton.h"
#include "XmlColor.h"
#include "XmlDocument.h"
#include "XmlOptions.h"

#include <QFile>

#include <QApplication>
#include <QCursor>
#include <QDesktopWidget>
#include <QHeaderView>
#include <QImage>
#include <QLayout>
#include <QPixmap>
#include <QMouseEvent>
#include <QPushButton>
#include <QShortcut>
#include <QSplitter>

//_____________________________________________________________
MainWindow::MainWindow( QWidget* parent ):
    BaseMainWindow( parent ),
    selectionFrame_( 0 )
{
    Debug::Throw( "MainWindow::MainWindow.\n" );
    setOptionName( "MAIN_WINDOW" );

    // central widget
    QWidget* main = new QWidget( this );
    QVBoxLayout *mainLayout = new QVBoxLayout();
    mainLayout->setMargin(2);
    mainLayout->setSpacing(5);
    main->setLayout( mainLayout );
    setCentralWidget( main );

    // actions
    _installActions();
    Application& application( *Singleton::get().application<Application>() );
    addAction( &application.closeAction() );

    // create colorSelectionFrame
    mainLayout->addWidget( selectionFrame_ = new ColorSelectionFrame( this ) );
    addAction( &selectionFrame_->saveAction() );
    connect( &selectionFrame_->saveAction(), SIGNAL( triggered() ), SLOT( _add() ) );

    // list
    mainLayout->addWidget( list_ = new ColorList(this) );
    list_->list().header()->hide();
    addAction( &list_->removeAction() );

    // connections
    connect( list_->list().selectionModel(), SIGNAL( currentChanged(const QModelIndex &, const QModelIndex & ) ), SLOT( _selectFromList(const QModelIndex &) ) );
    connect( &toggleListAction(), SIGNAL( toggled( bool ) ), SLOT( _toggleShowList( bool ) ) );

    // configuration update
    connect( &application, SIGNAL( configurationChanged() ), SLOT( _updateConfiguration() ) );
    connect( &application, SIGNAL( saveConfiguration() ), SLOT( _write() ) );

    // menu bar
    setMenuBar( menu_ = new Menu( this ) );
    _read();

}

//_____________________________________________________________
void MainWindow::closeEvent( QCloseEvent *event )
{
    Debug::Throw( "MainWindow::closeEvent.\n" );
    _write();
    return BaseMainWindow::closeEvent( event );
}

//_______________________________________
void MainWindow::_updateConfiguration( void )
{
    Debug::Throw( "MainWindow::_updateConfiguration.\n" );

    // resize
    toggleListAction().setChecked( XmlOptions::get().get<bool>( "SHOW_LIST" ) );

}

//_____________________________________________________________
void MainWindow::_read( void )
{

    Debug::Throw( "MainWindow::_read\n" );
    File file( XmlOptions::get().raw("RC_FILE") );

    // make sure file is hidden (windows only)
    if( file.localName().startsWith( '.' ) ) file.setHidden();

    // parse the file
    XmlDocument document;
    {
        QFile qfile( file );
        if( !document.setContent( &qfile ) )
        {
            Debug::Throw() << document.error() << endl;
            return;
        }
    }

    // look for relevant element
    QDomNodeList topNodes = document.elementsByTagName( BASE::XML::COLOR_LIST );
    if( topNodes.isEmpty() ) return;
    for(QDomNode node = topNodes.at(0).firstChild(); !node.isNull(); node = node.nextSibling() )
    {
        QDomElement element = node.toElement();
        if( element.isNull() ) continue;

        QString tagName( element.tagName() );
        if( tagName == BASE::XML::COLOR )
        {
            XmlColor color( element );
            if( color.isValid() ) list_->model().add( color );
        }
    }

    // set index selected
    if( list_->model().rowCount() )
    {
        QModelIndex selectedIndex( list_->model().index( 0, 0 ) );
        list_->list().selectionModel()->select( selectedIndex,  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
        _selectFromList( selectedIndex );
    }

    return;
}

//_____________________________________________________________
void MainWindow::_write( void )
{

    Debug::Throw( "MainWindow::_write\n" );

    // output file
    QString file( XmlOptions::get().raw("RC_FILE") );

    // create document and read
    XmlDocument document;
    {
        QFile qtfile( file );
        document.setContent( &qtfile );
    }

    // create main element
    QDomElement top = document.createElement( BASE::XML::COLOR_LIST );

    // retrieve colors from model
    ColorModel::List colors( list_->model().get() );
    foreach( const BASE::Color& color, colors )
    { top.appendChild( XmlColor( color ).domElement( document ) ); }

    // append top node to document and write
    document.replaceChild( top );
    {
        QFile qfile( file );
        if( qfile.open( QIODevice::WriteOnly ) )
        { qfile.write( document.toByteArray() ); }
    }

}

//_____________________________________________________________
void MainWindow::_add( void )
{
    Debug::Throw( "MainWindow::_add.\n" );
    list_->add( selectionFrame_->color() );
}

//____________________________________________________________
void MainWindow::_selectFromList( const QModelIndex &index )
{

    Debug::Throw( "MainWindow::_selectFromList\n" );
    if( index.isValid() )
    { selectionFrame_->setColor( list_->model().get( index ) ); }

}

//_____________________________________________________________
void MainWindow::_toggleShowList( bool state )
{

    Debug::Throw( "MainWindow::_toggleShowList.\n" );
    list_->setVisible( state );
    XmlOptions::get().set<bool>( "SHOW_LIST", state );

}

//_________________________________________________________________________
void MainWindow::_installActions( void )
{
    Debug::Throw( "MainWindow::_installActions.\n" );

    // toggle list action
    addAction( toggleListAction_ = new QAction( "&Show List", this ) );
    toggleListAction_->setToolTip( "Show/hide color list" );
    toggleListAction_->setCheckable( true );
    toggleListAction_->setChecked( true );

}
